#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <errno.h>

#include <assert.h>

void help(void)
{
	puts("This tool converts binary data to ASCII file ready for inclusion in C program.");
	puts("Usage: bin2c <input file> [output file] [c_array_name] [str_len]");
}

int do_convert(FILE *in,FILE *out,char *aname,int str_len)
{
	int i, len, c, nb, bpl;
	fseek(in,0,SEEK_END);
	len = ftell(in);
	fseek(in,0,SEEK_SET);
	if(!len)
	{
		fprintf(stderr,"Empty input file.\n");
		return -1;
	}
	fprintf(out,"unsigned char %s[%d] =%c{%c    ",aname,len,0x0A,0x0A);
	bpl = (str_len - 4) / 5;
	len--;
	for(i = nb = 0;i < len;i++)
	{
		c = fgetc(in);
		if(c == EOF)
		{
			fprintf(stderr,"Unexpected end of input file.\n");
			return -1;
		}
		fprintf(out,"0x%02X,",c);
		nb++;
		if(nb == bpl)
		{
			fprintf(out,"%c    ",0x0A);
			nb = 0;
		}
	}
	c = fgetc(in);
	if(c == EOF)
	{
		fprintf(stderr,"Unexpected end of input file.\n");
		return -1;
	}
	fprintf(out,"0x%02X%c};%c",c,0x0A,0x0A);
	return 0;
}

int main(int argc, char *argv[])
{
	int result, str_len;
	char oname[256], *aname;
	FILE *in, *out;
	if(argc < 2 || argc > 5)
	{
		help();
		return -1;
	}
	if(argc == 2)
	{
		if(strlen(argv[1]) > 253)
		{
			puts("Input file name too long");
			return -1;
		}
	}
	else if(strlen(argv[2]) > 255)
	{
		puts("Output file name too long");
		return -1;
	}
	in = fopen(argv[1],"rb");
	if(!in)
	{
		fprintf(stderr,"Unable to open \"%s\": %s.\n",argv[1],strerror(errno));
		return -1;
	}
	if(argc == 2)
	{
		strcpy(oname,argv[1]);
		strcat(oname,".c");
	}
	else
		strcpy(oname,argv[2]);
	out = fopen(oname,"wb");
	if(!out)
	{
		fprintf(stderr,"Unable to create \"%s\": %s.\n",oname,strerror(errno));
		return -1;
	}
	aname = (argc > 3) ? argv[3] : argv[1];
	if(argc == 5)
	{
		str_len = atoi(argv[4]);
		if(str_len < 9)
		{
			fprintf(stderr,"Bad string len: \"%s\".",argv[4]);
			str_len = 75;
		}
	}
	else
		str_len = 75;
	result = do_convert(in,out,aname,str_len);
	fclose(in);
	fclose(out);
	return result;
}
